==> UDFclient.0.3.3.tgz: UDFclient.0.3.3/cdrw_format.c <==
/* $NetBSD$ */

/*
 * File "cdrw_format.c" is part of the UDFclient toolkit.
 * File $Id: cdrw_format.c,v 1.5 2005/01/11 20:51:34 imago Exp $ $Name:  $
 *
 * Copyright (c) 2003, 2004 Reinoud Zandijk <reinoud@netbsd.org>
 * All rights reserved.
 *
 * The UDFclient toolkit is distributed under the Clarified Artistic Licence.
 * A copy of the licence is included in the distribution as
 * `LICENCE.clearified.artistic' and a copy of the licence can also be
 * requested at the GNU foundantion's website.
 *
 * Visit the UDFclient toolkit homepage http://www.13thmonkey.org/udftoolkit/
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <strings.h>
#include <assert.h>
#include <limits.h>
#include <sys/types.h>
#include <inttypes.h>

#include "scsi_sense.h"
#include "scsilib.h"


/* globals */
struct udf_discinfo *disc;
extern int udf_verbose;
extern int scsilib_verbose;


extern void udf_dump_discinfo(struct udf_discinfo *disc);


/* format a CD-RW disc */
int udf_discinfo_format_cdrw(struct udf_discinfo * disc, uint32_t blocks) {
	scsicmd cmd;
	uint8_t buffer[16];
	int error;

	bzero(cmd, SCSI_CMD_LEN);
	bzero(buffer, sizeof(buffer));

	cmd[0] = 0x04;				/* format unit 					*/
	cmd[1] = 0x17;				/* parameter list format 7 follows		*/
	cmd[5] = 0;				/* control					*/

	/* format list header */
	buffer[ 0] = 0;				/* reserved					*/
	buffer[ 1] = 0;				/* format options not valid (use defaults)	*/
/*	buffer[ 1] |= (1 << 1); */		/* immediate return 				*/
	buffer[ 2] = 0;				/* MSB format descriptor length			*/
	buffer[ 3] = 8;				/* LSB ...					*/

	/* for CD-RW the initialisation pattern bit is reserved, but there IS one */
	buffer[ 4] = 0;				/* no header					*/
	buffer[ 5] = 0;				/* default pattern				*/
	buffer[ 6] = 0;				/* pattern length MSB				*/
	buffer[ 7] = 0;				/* pattern length LSB				*/

	/* 8 bytes of format descriptor */
	/* (s)ession bit 1<<7, (g)row bit 1<<6  */
	/* SG	action	*/
	/* 00	format disc with number of user data blocks	*/
	/* 10	create new session with number of data blocks	*/
	/* x1	grow session to be number of data blocks	*/

	buffer[ 8] = 0x00;			/* session and grow bits (7 and 6)		*/
	buffer[ 9] = 0;				/* reserved */
	buffer[10] = 0;				/* reserved */
	buffer[11] = 0;				/* reserved */
	buffer[12] = (blocks >> 24) & 0xff;	/* blocks MSB					*/
	buffer[13] = (blocks >> 16) & 0xff;
	buffer[14] = (blocks >>  8) & 0xff;
	buffer[15] = (blocks      ) & 0xff;	/* blocks LSB					*/

	/* this will take a while .... */
	error = scsi_command(disc, cmd, 6, buffer, sizeof(buffer), UINT_MAX, SCSI_WRITECMD);
	if (error) {
		fprintf(stderr, "format unit returned : %s\n", strerror(error));
		return error;
	};

	return 0;
}


int udf_discinfo_blank_cdrw(struct udf_discinfo *disc) {
	scsicmd cmd;
	int error;

	bzero(cmd, SCSI_CMD_LEN);
	cmd[ 0] = 0xA1;				/* blank		*/
	cmd[ 1] = 0;				/* blank complete 	*/
	cmd[11] = 0;				/* control		*/
	error = scsi_command(disc, cmd, 12, NULL, 0, UINT_MAX, SCSI_WRITECMD);

	return error;
}


int usage(char *program) {
	fprintf(stderr, "Usage: %s [options] devicename\n", program);
	fprintf(stderr, "-B		blank cd-rw disc before formatting\n"
			"-F		format cd-rw disc\n"
			"-s 		verbose SCSI command errors\n"
	);
	return 1;
}


extern char	*optarg;
extern int	 optind;
extern int	 optreset;


int main(int argc, char *argv[]) {
	struct scsi_addr saddr;
	uint32_t capacity;
	char *progname, *dev_name;
	int blank, format;
	int flag, error;

	progname = strdup(argv[0]);
	if (argc == 1) {
		return usage(progname);
	};

	scsilib_verbose = 0;
	blank           = 0;
	format          = 0;
	while ((flag = getopt(argc, argv, "BFs")) != -1) {
		switch (flag) {
			case 'B' :
				blank = 1;
				break;
			case 'F' :
				format = 1;
				break;
			case 's' :
				scsilib_verbose = 1;
				break;
			default :
				return usage(progname);
		};
	};
	argv += optind;
	argc -= optind;

	if (!blank && !format) {
		fprintf(stderr, "%s : at least one of -B or -F needs to be specified\n\n", progname);
		return usage(progname);
	};

	if (argc != 1) return usage(progname);

	dev_name = strdup(*argv);

	/* Open the device */
	printf("Opening device %s\n", dev_name);
	error = udf_open_disc(dev_name, &disc);
	if (error) {
		fprintf(stderr, "Device failed to open : %s\n", strerror(error));
		exit(1);
	};

	error = scsi_check_for_scsi(disc);
	if (error) {
		fprintf(stderr, "sorry, not a SCSI device : %s\n", strerror(error));
		exit(1);
	};

	error = scsi_identify(disc, &saddr);
	if (error) {
		fprintf(stderr, "SCSI identify returned : %s\n", strerror(error));
		exit(1);
	};

	printf("\nDevice dentifies itself as : ");
	if (saddr.type == 0) {
		printf("SCSI   busnum = %d, target = %d, lun = %d\n", saddr.addr.scsi.scbus, saddr.addr.scsi.target, saddr.addr.scsi.lun);
	} else {
		printf("ATAPI  busnum = %d, drive = %d\n", saddr.addr.atapi.atbus, saddr.addr.atapi.drive);
	};
	printf("\n");

	/* can we format it? */
	if (disc->devdrv_class != UDF_DEVDRV_TYPE_CD_RW) {
		printf("Euhm... this is not a CD rewritable disc\n");
		udf_dump_discinfo(disc);
		udf_close_disc(disc);

		return 1;
	};

	if (blank) {
		printf("\nBlanking disc ... (can take a few minutes) ..."); fflush(stdout);
		error = udf_discinfo_blank_cdrw(disc);

		if (error) {
			printf("fail\n"); fflush(stdout);
			fprintf(stderr, "Blanking failed because of : %s\n", strerror(error));
			udf_close_disc(disc);

			return 1;
		} else {
			printf("success!\n\n");

			udf_close_disc(disc);
			free(disc);		/* nessisary? */

			printf("Re-opening disc\n\n");
			/* reread disc info */
			error = udf_open_disc(dev_name, &disc);
			if (error) {
				fprintf(stderr, "Failure to re-open disc for updated info : %s\n", strerror(error));
				return 1;
			};
			udf_dump_discinfo(disc);
		};
	};

	if (format) {
		capacity = disc->session_end[disc->num_sessions-1];

		printf("CD-RW has a capacity of %" PRIu32 " blocks (%" PRIu64 " bytes)\n", capacity, (uint64_t) capacity*CD_SECTOR_LENGTH);
		printf("\nFormatting.... (please be patient (+/- 10 minutes)) ..."); fflush(stdout);

		error = udf_discinfo_format_cdrw(disc, capacity);
		if (error) {
			printf("fail\n"); fflush(stdout);
			fprintf(stderr, "Formatting failed because of : %s\n", strerror(error));
		} else {
			printf("success!\n\n");
			udf_dump_discinfo(disc);
		};
	};

	udf_close_disc(disc);

	return error;
}

==> UDFclient.0.4.9.tgz: UDFclient.0.4.9/cdrw_format.c <==
/* $NetBSD$ */

/*
 * File "cdrw_format.c" is part of the UDFclient toolkit.
 * File $Id: cdrw_format.c,v 1.10 2005/06/23 00:36:15 imago Exp $ $Name:  $
 *
 * Copyright (c) 2003, 2004, 2005 Reinoud Zandijk <reinoud@netbsd.org>
 * All rights reserved.
 *
 * The UDFclient toolkit is distributed under the Clarified Artistic Licence.
 * A copy of the licence is included in the distribution as
 * `LICENCE.clearified.artistic' and a copy of the licence can also be
 * requested at the GNU foundantion's website.
 *
 * Visit the UDFclient toolkit homepage http://www.13thmonkey.org/udftoolkit/
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <strings.h>
#include <assert.h>
#include <limits.h>
#include <sys/types.h>
#include <inttypes.h>

#include "scsi_sense.h"
#include "scsilib.h"


/* globals */
struct udf_discinfo *disc;
extern int udf_verbose;
extern int scsilib_verbose;


extern void udf_dump_discinfo(struct udf_discinfo *disc);


/* format a CD-RW disc */
int udf_discinfo_format_cdrw(struct udf_discinfo * disc, uint32_t blocks) {
	scsicmd cmd;
	uint8_t buffer[16];
	int error;

	bzero(cmd, SCSI_CMD_LEN);
	bzero(buffer, sizeof(buffer));

	cmd[0] = 0x04;				/* format unit 					*/
	cmd[1] = 0x17;				/* parameter list format 7 follows		*/
	cmd[5] = 0;				/* control					*/

	/* format list header */
	buffer[ 0] = 0;				/* reserved					*/
	buffer[ 1] = 0;				/* format options not valid (use defaults)	*/
/*	buffer[ 1] |= (1 << 1); */		/* immediate return 				*/
	buffer[ 2] = 0;				/* MSB format descriptor length			*/
	buffer[ 3] = 8;				/* LSB ...					*/

	/* for CD-RW the initialisation pattern bit is reserved, but there IS one */
	buffer[ 4] = 0;				/* no header					*/
	buffer[ 5] = 0;				/* default pattern				*/
	buffer[ 6] = 0;				/* pattern length MSB				*/
	buffer[ 7] = 0;				/* pattern length LSB				*/

	/* 8 bytes of format descriptor */
	/* (s)ession bit 1<<7, (g)row bit 1<<6  */
	/* SG	action	*/
	/* 00	format disc with number of user data blocks	*/
	/* 10	create new session with number of data blocks	*/
	/* x1	grow session to be number of data blocks	*/

	buffer[ 8] = 0x00;			/* session and grow bits (7 and 6)		*/
	buffer[ 9] = 0;				/* reserved */
	buffer[10] = 0;				/* reserved */
	buffer[11] = 0;				/* reserved */
	buffer[12] = (blocks >> 24) & 0xff;	/* blocks MSB					*/
	buffer[13] = (blocks >> 16) & 0xff;
	buffer[14] = (blocks >>  8) & 0xff;
	buffer[15] = (blocks      ) & 0xff;	/* blocks LSB					*/

	/* this will take a while .... */
	error = scsi_command(disc, cmd, 6, buffer, sizeof(buffer), UINT_MAX, SCSI_WRITECMD);
	if (error) {
		fprintf(stderr, "format unit returned : %s\n", strerror(error));
		return error;
	};

	return 0;
}


int udf_discinfo_blank_cdrw(struct udf_discinfo *disc) {
	scsicmd cmd;
	int error;

	bzero(cmd, SCSI_CMD_LEN);
	cmd[ 0] = 0xA1;				/* blank		*/
	cmd[ 1] = 0;				/* blank complete 	*/
	cmd[11] = 0;				/* control		*/
	error = scsi_command(disc, cmd, 12, NULL, 0, UINT_MAX, SCSI_WRITECMD);

	return error;
}


int usage(char *program) {
	fprintf(stderr, "Usage: %s [options] devicename\n", program);
	fprintf(stderr, "-B		blank cd-rw disc before formatting\n"
			"-F		format cd-rw disc\n"
			"-D 		verbose SCSI command errors\n"
	);
	return 1;
}


extern char	*optarg;
extern int	 optind;
extern int	 optreset;


int main(int argc, char *argv[]) {
	struct uscsi_addr saddr;
	uint32_t capacity;
	char *progname, *dev_name;
	int blank, format;
	int flag, error;

	progname = strdup(argv[0]);
	if (argc == 1) {
		return usage(progname);
	};

	scsilib_verbose = 0;
	blank           = 0;
	format          = 0;
	while ((flag = getopt(argc, argv, "BFD")) != -1) {
		switch (flag) {
			case 'B' :
				blank = 1;
				break;
			case 'F' :
				format = 1;
				break;
			case 'D' :
				scsilib_verbose = 1;
				break;
			default :
				return usage(progname);
		};
	};
	argv += optind;
	argc -= optind;

	if (!blank && !format) {
		fprintf(stderr, "%s : at least one of -B or -F needs to be specified\n\n", progname);
		return usage(progname);
	};

	if (argc != 1) return usage(progname);

	dev_name = strdup(*argv);

	/* Open the device */
	printf("Opening device %s\n", dev_name);
	error = udf_open_disc(dev_name, &disc);
	if (error) {
		fprintf(stderr, "Device failed to open : %s\n", strerror(error));
		exit(1);
	};

	error = scsi_check_for_scsi(disc);
	if (error) {
		fprintf(stderr, "sorry, not a SCSI device : %s\n", strerror(error));
		exit(1);
	};

	error = scsi_identify(disc, &saddr);
	if (error) {
		fprintf(stderr, "SCSI identify returned : %s\n", strerror(error));
		exit(1);
	};

	printf("\nDevice dentifies itself as : ");

	if (saddr.type == USCSI_TYPE_SCSI) {
		printf("SCSI   busnum = %d, target = %d, lun = %d\n", saddr.addr.scsi.scbus, saddr.addr.scsi.target, saddr.addr.scsi.lun);
	} else {
		printf("ATAPI  busnum = %d, drive = %d\n", saddr.addr.atapi.atbus, saddr.addr.atapi.drive);
	};

	printf("\n");

	/* can we format it? */
	if (disc->devdrv_class != UDF_DEVDRV_TYPE_CD_RW) {
		printf("Euhm... this is not a CD rewritable disc\n");
		udf_dump_discinfo(disc);
		udf_close_disc(disc);

		return 1;
	};

	if (blank && !disc->erasable) {
		printf("\nDrive is reporting it can't erase/blank discs... try another drive\n");
		udf_dump_discinfo(disc);
		udf_close_disc(disc);

		return 1;
	};

	if (blank) {
		printf("\nBlanking disc ... (can take a few minutes) ..."); fflush(stdout);
		error = udf_discinfo_blank_cdrw(disc);

		if (error) {
			printf("fail\n"); fflush(stdout);
			fprintf(stderr, "Blanking failed because of : %s\n", strerror(error));
			udf_close_disc(disc);

			return 1;
		} else {
			printf("success!\n\n");

			udf_close_disc(disc);
			free(disc);		/* nessisary? */

			printf("Re-opening disc\n\n");
			/* reread disc info */
			error = udf_open_disc(dev_name, &disc);
			if (error) {
				fprintf(stderr, "Failure to re-open disc for updated info : %s\n", strerror(error));
				return 1;
			};
			udf_dump_discinfo(disc);
		};
	};

	if (format) {
		capacity = disc->session_end[disc->num_sessions-1];

		printf("CD-RW has a capacity of %" PRIu32 " blocks (%" PRIu64 " bytes)\n", capacity, (uint64_t) capacity * CD_SECTOR_SIZE);
		printf("\nFormatting.... (please be patient (+/- 10 minutes)) ..."); fflush(stdout);

		error = udf_discinfo_format_cdrw(disc, capacity);
		if (error) {
			printf("fail\n"); fflush(stdout);
			fprintf(stderr, "Formatting failed because of : %s\n", strerror(error));
		} else {
			printf("success!\n\n");
			udf_dump_discinfo(disc);
		};
	};

	udf_close_disc(disc);

	return error;
}
